/****************************************************************************
* FILE: poe_bandwidth.c
* DESCRIPTION:
*   This code conducts timing tests on messages sent between 2 MPI tasks.
*   It sends/receives N roundtrips of incrementally sized messages
*   from start size J to finish size K by increment I.
* LAST REVISED:  06/04/06 Blaise Barney
******************************************************************************/

#include "mpi.h"
#include <stdio.h>

#define MAXSIZE         2000000
#define task0           0
#define task1           1
#define tag0            0
#define tag1            1

int main (int argc, char* argv[])
{

int     numtasks, rank, n, i, rndtrps, start, end, incr, rc, length;
double  t1, t2, nbytes, etime, bandwidth;
char    c[MAXSIZE], host[MPI_MAX_PROCESSOR_NAME],
        host1[MPI_MAX_PROCESSOR_NAME];
MPI_Status Stat;

/* Some initializations and error checking */
MPI_Init(&argc,&argv);
MPI_Comm_size(MPI_COMM_WORLD, &numtasks);
if (numtasks < 2) {
  printf("ERROR: Number of tasks= %d. Need two tasks. ",numtasks);
  printf("Quitting...\n");
  MPI_Abort(MPI_COMM_WORLD, rc);
  exit(0);
  }
else if (numtasks > 2) {
  printf("Number of tasks= %d. Only need 2 tasks. ",numtasks);
  printf("Ignoring extra tasks...\n");
  }
MPI_Comm_rank(MPI_COMM_WORLD, &rank);
MPI_Get_processor_name(host, &length);
start = 100000;
end = MAXSIZE;
incr = 100000;
rndtrps = 1000;

/**************************** task 0 ***********************************/
if (rank == task0) {
  for (i=0; i<MAXSIZE; i++)
    c[i] = 'x';
  MPI_Recv(&host1, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, task1, tag1, 
           MPI_COMM_WORLD, &Stat);

  printf("\n****** MPI/POE Bandwidth Test ******\n");
  printf("Message start size= %d bytes\n",start);
  printf("Message finish size= %d bytes\n",end);
  printf("Incremented by %d bytes per iteration\n",incr);
  printf("Roundtrips per iteration= %d\n",rndtrps);
  printf("Task 0 running on: %s\n",host);
  printf("Task 1 running on: %s\n\n",host1);
  printf("Message Size   Bandwidth (bytes/sec)\n");

  for (n=start; n<=end; n=n+incr) {
    etime = 0.0;
    t1 = MPI_Wtime();
    for (i=1; i<=rndtrps; i++){
      MPI_Send(&c, n, MPI_CHAR, task1, tag0, MPI_COMM_WORLD);
      MPI_Recv(&c, n, MPI_CHAR, task1, tag1, MPI_COMM_WORLD, &Stat);
      }
    t2 = MPI_Wtime();
    etime = t2 - t1;
    nbytes =  2.0 * (n * rndtrps);
    bandwidth = nbytes/etime;
    printf("%9d %16e\n",n, bandwidth);
    }
  }   /* end of task 0 */


/****************************  task 1 ************************************/
if (rank == task1) {
  MPI_Send(&host, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, task0, tag1, 
           MPI_COMM_WORLD);
  for (n=start; n<=end; n=n+incr) {
    for (i=1; i<=rndtrps; i++){
      MPI_Recv(&c, n, MPI_CHAR, task0, tag0, MPI_COMM_WORLD, &Stat);
      MPI_Send(&c, n, MPI_CHAR, task0, tag1, MPI_COMM_WORLD);
      }
    }
  }  /* end of task 1 */


MPI_Finalize();
}  /* end of main */
